@@ -9,11 +9,11 @@
 #include "qmediadevices.h"
 #include "qaudiobuffer.h"
 #include <QtCore/qloggingcategory.h>
-#include <private/qplatformmediadevices_p.h>
+#include <private/qplatformaudiodevices_p.h>
 #include <private/qplatformmediaintegration_p.h>
 #include <private/qplatformaudioresampler_p.h>
 
-static Q_LOGGING_CATEGORY(qLcSoundEffect, "qt.multimedia.soundeffect")
+Q_STATIC_LOGGING_CATEGORY(qLcSoundEffect, "qt.multimedia.soundeffect")
 
 QT_BEGIN_NAMESPACE
 
@@ -70,10 +70,11 @@ public:
     void setLoopsRemaining(int loopsRemaining);
     void setStatus(QSoundEffect::Status status);
     void setPlaying(bool playing);
+    bool updateAudioOutput();
 
 public Q_SLOTS:
-    void sampleReady();
-    void decoderError();
+    void sampleReady(QSample *);
+    void decoderError(QSample *);
     void stateChanged(QAudio::State);
 
 public:
@@ -100,62 +101,26 @@ QSoundEffectPrivate::QSoundEffectPrivate(QSoundEffect *q, const QAudioDevice &au
 {
     open(QIODevice::ReadOnly);
 
-    QPlatformMediaIntegration::instance()->mediaDevices()->prepareAudio();
+    QPlatformMediaIntegration::instance()->audioDevices()->prepareAudio();
 }
 
-void QSoundEffectPrivate::sampleReady()
+void QSoundEffectPrivate::sampleReady(QSample *sample)
 {
+    if (sample && sample != m_sample.get())
+        return;
+
     if (m_status == QSoundEffect::Error)
         return;
 
     qCDebug(qLcSoundEffect) << this << "sampleReady: sample size:" << m_sample->data().size();
     disconnect(m_sample.get(), &QSample::error, this, &QSoundEffectPrivate::decoderError);
     disconnect(m_sample.get(), &QSample::ready, this, &QSoundEffectPrivate::sampleReady);
+
     if (!m_audioSink) {
-        const auto audioDevice =
-                m_audioDevice.isNull() ? QMediaDevices::defaultAudioOutput() : m_audioDevice;
-
-        if (audioDevice.isNull()) {
-            // We are likely on a virtual machine, for example in CI
-            qCCritical(qLcSoundEffect) << "Failed to play sound. No audio devices present.";
-            setStatus(QSoundEffect::Error);
-            return;
-        }
-
-        const auto &sampleFormat = m_sample->format();
-        const auto sampleChannelConfig =
-                sampleFormat.channelConfig() == QAudioFormat::ChannelConfigUnknown
-                ? QAudioFormat::defaultChannelConfigForChannelCount(sampleFormat.channelCount())
-                : sampleFormat.channelConfig();
-
-        if (sampleChannelConfig != audioDevice.channelConfiguration()
-            && audioDevice.channelConfiguration() != QAudioFormat::ChannelConfigUnknown) {
-            qCDebug(qLcSoundEffect) << "Create resampler for channels mapping: config"
-                                    << sampleFormat.channelConfig() << "=> config"
-                                    << audioDevice.channelConfiguration();
-            auto outputFormat = sampleFormat;
-            outputFormat.setChannelConfig(audioDevice.channelConfiguration());
-
-            const auto resampler = QPlatformMediaIntegration::instance()->createAudioResampler(
-                    m_sample->format(), outputFormat);
-            if (resampler)
-                m_audioBuffer = resampler.value()->resample(m_sample->data().constData(),
-                                                            m_sample->data().size());
-            else
-                qCDebug(qLcSoundEffect) << "Cannot create resampler for channels mapping";
-        }
-
-        if (!m_audioBuffer.isValid())
-            m_audioBuffer = QAudioBuffer(m_sample->data(), m_sample->format());
-
-        m_audioSink.reset(new QAudioSink(audioDevice, m_audioBuffer.format()));
-
-        connect(m_audioSink.get(), &QAudioSink::stateChanged, this, &QSoundEffectPrivate::stateChanged);
-        if (!m_muted)
-            m_audioSink->setVolume(m_volume);
-        else
-            m_audioSink->setVolume(0);
+        if (!updateAudioOutput()) // Create audio sink
+            return; // Returns if no audio devices are available
     }
+
     m_sampleReady = true;
     setStatus(QSoundEffect::Ready);
 
@@ -165,8 +130,11 @@ void QSoundEffectPrivate::sampleReady()
     }
 }
 
-void QSoundEffectPrivate::decoderError()
+void QSoundEffectPrivate::decoderError(QSample *sample)
 {
+    if (sample && sample != m_sample.get())
+        return;
+
     qWarning("QSoundEffect(qaudio): Error decoding source %ls", qUtf16Printable(m_url.toString()));
     disconnect(m_sample.get(), &QSample::ready, this, &QSoundEffectPrivate::sampleReady);
     disconnect(m_sample.get(), &QSample::error, this, &QSoundEffectPrivate::decoderError);
@@ -181,6 +149,63 @@ void QSoundEffectPrivate::stateChanged(QAudio::State state)
         q_ptr->stop();
 }
 
+bool QSoundEffectPrivate::updateAudioOutput()
+{
+    const auto audioDevice =
+            m_audioDevice.isNull() ? QMediaDevices::defaultAudioOutput() : m_audioDevice;
+
+    if (audioDevice.isNull()) {
+        // We are likely on a virtual machine, for example in CI
+        qCCritical(qLcSoundEffect) << "Failed to update audio output. No audio devices available.";
+        setStatus(QSoundEffect::Error);
+        return false;
+    }
+
+    if (m_audioDevice.isNull()) {
+        q_ptr->setAudioDevice(audioDevice); // Updates m_audioDevice and emits audioDeviceChanged
+    }
+
+    m_audioBuffer = {};
+
+    Q_ASSERT(m_sample);
+
+    const auto &sampleFormat = m_sample->format();
+    const auto sampleChannelConfig =
+            sampleFormat.channelConfig() == QAudioFormat::ChannelConfigUnknown
+            ? QAudioFormat::defaultChannelConfigForChannelCount(sampleFormat.channelCount())
+            : sampleFormat.channelConfig();
+
+    if (sampleChannelConfig != audioDevice.channelConfiguration()
+        && audioDevice.channelConfiguration() != QAudioFormat::ChannelConfigUnknown) {
+        qCDebug(qLcSoundEffect) << "Create resampler for channels mapping: config"
+                                << sampleFormat.channelConfig() << "=> config"
+                                << audioDevice.channelConfiguration();
+        auto outputFormat = sampleFormat;
+        outputFormat.setChannelConfig(audioDevice.channelConfiguration());
+
+        const auto resampler = QPlatformMediaIntegration::instance()->createAudioResampler(
+                m_sample->format(), outputFormat);
+        if (resampler)
+            m_audioBuffer = resampler.value()->resample(m_sample->data().constData(),
+                                                        m_sample->data().size());
+        else
+            qCDebug(qLcSoundEffect) << "Cannot create resampler for channels mapping";
+    }
+
+    if (!m_audioBuffer.isValid())
+        m_audioBuffer = QAudioBuffer(m_sample->data(), m_sample->format());
+
+    m_audioSink.reset(new QAudioSink(audioDevice, m_audioBuffer.format()));
+
+    connect(m_audioSink.get(), &QAudioSink::stateChanged, this, &QSoundEffectPrivate::stateChanged);
+    if (!m_muted)
+        m_audioSink->setVolume(m_volume);
+    else
+        m_audioSink->setVolume(0);
+
+    return true;
+}
+
 qint64 QSoundEffectPrivate::readData(char *data, qint64 len)
 {
     qCDebug(qLcSoundEffect) << this << "readData" << len << m_runningCount;
@@ -197,14 +222,14 @@ qint64 QSoundEffectPrivate::readData(char *data, qint64 len)
     const char *sampleData = m_audioBuffer.constData<char>();
 
     while (len && m_runningCount) {
-        int toWrite = qMin(sampleSize - m_offset, len);
+        int toWrite = qMax(0, qMin(sampleSize - m_offset, len));
         memcpy(data, sampleData + m_offset, toWrite);
         bytesWritten += toWrite;
         data += toWrite;
         len -= toWrite;
         m_offset += toWrite;
         if (m_offset >= sampleSize) {
-            if (m_runningCount > 0 && m_runningCount != QSoundEffect::Infinite)
+            if (m_runningCount > 0)
                 setLoopsRemaining(m_runningCount - 1);
             m_offset = 0;
         }
@@ -245,7 +270,7 @@ void QSoundEffectPrivate::setPlaying(bool playing)
 {
     qCDebug(qLcSoundEffect) << this << "setPlaying(" << playing << ")" << m_playing;
     if (m_audioSink) {
-        m_audioSink->stop();
+        m_audioSink->reset();
         if (playing && !m_sampleReady)
             return;
     }
@@ -294,7 +319,7 @@ void QSoundEffectPrivate::setPlaying(bool playing)
 
 /*!
     \qmltype SoundEffect
-    \instantiates QSoundEffect
+    \nativetype QSoundEffect
     \brief The SoundEffect type provides a way to play sound effects in QML.
 
     \inmodule QtMultimedia
@@ -403,7 +428,6 @@ void QSoundEffect::setSource(const QUrl &url)
     stop();
 
     d->m_url = url;
-
     d->m_sampleReady = false;
 
     if (url.isEmpty()) {
@@ -421,8 +445,7 @@ void QSoundEffect::setSource(const QUrl &url)
             disconnect(d->m_sample.get(), &QSample::error, d, &QSoundEffectPrivate::decoderError);
             disconnect(d->m_sample.get(), &QSample::ready, d, &QSoundEffectPrivate::sampleReady);
         }
-        d->m_sample->release();
-        d->m_sample = nullptr;
+        d->m_sample.reset();
     }
 
     if (d->m_audioSink) {
@@ -437,10 +460,10 @@ void QSoundEffect::setSource(const QUrl &url)
 
     switch (d->m_sample->state()) {
     case QSample::Ready:
-        d->sampleReady();
+        d->sampleReady(d->m_sample.get());
         break;
     case QSample::Error:
-        d->decoderError();
+        d->decoderError(d->m_sample.get());
         break;
     default:
         break;
@@ -523,10 +546,28 @@ QAudioDevice QSoundEffect::audioDevice()
 
 void QSoundEffect::setAudioDevice(const QAudioDevice &device)
 {
+    qCDebug(qLcSoundEffect) << this << "setAudioDevice:" << device.description();
+
     if (d->m_audioDevice == device)
         return;
-    // ### recreate the QAudioSink if needed
+
     d->m_audioDevice = device;
+
+    if (!d->m_sampleReady) {
+        emit audioDeviceChanged();
+        return; // The audio sink will be recreated later by QSoundEffect::sampleReady()
+    }
+
+    bool playing = d->m_playing;
+    std::chrono::microseconds current_time{d->m_audioBuffer.format().durationForBytes(d->m_offset)};
+
+    // Recreate the QAudioSink with the new audio device and current sample
+    if (d->updateAudioOutput() && playing) {
+        // Resume playback from current position
+        d->m_offset = d->m_audioBuffer.format().bytesForDuration(current_time.count());
+        d->setPlaying(true);
+    }
+
     emit audioDeviceChanged();
 }
 
@@ -549,7 +590,7 @@ int QSoundEffect::loopsRemaining() const
 
 
 /*!
-    \qmlproperty qreal QtMultimedia::SoundEffect::volume
+    \qmlproperty real QtMultimedia::SoundEffect::volume
 
     This property holds the volume of the sound effect playback.
 
