@@ -13,8 +13,8 @@
 #include "qffmpegaudioinput_p.h"
 #include "qffmpegaudiodecoder_p.h"
 #include "qffmpegresampler_p.h"
-#include "qffmpegsymbolsresolve_p.h"
 #include "qgrabwindowsurfacecapture_p.h"
+#include "qffmpegconverter_p.h"
 
 #ifdef Q_OS_MACOS
 #include <VideoToolbox/VideoToolbox.h>
@@ -40,6 +40,7 @@
 #    include "qandroidvideodevices_p.h"
 #    include "qandroidcamera_p.h"
 #    include "qandroidimagecapture_p.h"
+#    include "qandroidscreencapture_p.h"
 extern "C" {
 #  include <libavutil/log.h>
 #  include <libavcodec/jni.h>
@@ -60,29 +61,19 @@ extern "C" {
 #include "qx11capturablewindows_p.h"
 #endif
 
+#if QT_CONFIG(pipewire_screencapture)
+#  include <QtMultimedia/private/qpipewire_screencapture_p.h>
+#endif
+
 #if QT_CONFIG(eglfs)
 #include "qeglfsscreencapture_p.h"
 #endif
 
+#include <QtCore/qloggingcategory.h>
+
 QT_BEGIN_NAMESPACE
 
-class QFFmpegMediaPlugin : public QPlatformMediaPlugin
-{
-    Q_OBJECT
-    Q_PLUGIN_METADATA(IID QPlatformMediaPlugin_iid FILE "ffmpeg.json")
-
-public:
-    QFFmpegMediaPlugin()
-      : QPlatformMediaPlugin()
-    {}
-
-    QPlatformMediaIntegration* create(const QString &name) override
-    {
-        if (name == QLatin1String("ffmpeg"))
-            return new QFFmpegMediaIntegration;
-        return nullptr;
-    }
-};
+Q_STATIC_LOGGING_CATEGORY(qLcFFmpeg, "qt.multimedia.ffmpeg");
 
 bool thread_local FFmpegLogsEnabledInThread = true;
 static bool UseCustomFFmpegLogger = false;
@@ -99,8 +90,8 @@ static void qffmpegLogCallback(void *ptr, int level, const char *fmt, va_list vl
     if (level < 0 || level > av_log_get_level())
         return;
 
-    QString message = QString("FFmpeg log: %1").arg(QString::vasprintf(fmt, vl));
-    if (message.endsWith("\n"))
+    QString message = QStringLiteral("FFmpeg log: %1").arg(QString::vasprintf(fmt, vl));
+    if (message.endsWith(u"\n"))
         message.removeLast();
 
     if (level == AV_LOG_DEBUG || level == AV_LOG_TRACE)
@@ -125,22 +116,22 @@ static void setupFFmpegLogger()
 
 static QPlatformSurfaceCapture *createScreenCaptureByBackend(QString backend)
 {
-    if (backend == QLatin1String("grabwindow"))
+    if (backend == u"grabwindow")
         return new QGrabWindowSurfaceCapture(QPlatformSurfaceCapture::ScreenSource{});
 
 #if QT_CONFIG(eglfs)
-    if (backend == QLatin1String("eglfs"))
+    if (backend == u"eglfs")
         return new QEglfsScreenCapture;
 #endif
 
 #if QT_CONFIG(xlib)
-    if (backend == QLatin1String("x11"))
+    if (backend == u"x11")
         return new QX11SurfaceCapture(QPlatformSurfaceCapture::ScreenSource{});
 #elif defined(Q_OS_WINDOWS)
-    if (backend == QLatin1String("dxgi"))
+    if (backend == u"dxgi")
         return new QFFmpegScreenCaptureDxgi;
 #elif defined(Q_OS_MACOS)
-    if (backend == QLatin1String("avf"))
+    if (backend == u"avf")
         return new QAVFScreenCapture;
 #endif
     return nullptr;
@@ -148,41 +139,41 @@ static QPlatformSurfaceCapture *createScreenCaptureByBackend(QString backend)
 
 static QPlatformSurfaceCapture *createWindowCaptureByBackend(QString backend)
 {
-    if (backend == QLatin1String("grabwindow"))
+    if (backend == u"grabwindow")
         return new QGrabWindowSurfaceCapture(QPlatformSurfaceCapture::WindowSource{});
 
 #if QT_CONFIG(xlib)
-    if (backend == QLatin1String("x11"))
+    if (backend == u"x11")
         return new QX11SurfaceCapture(QPlatformSurfaceCapture::WindowSource{});
 #elif defined(Q_OS_WINDOWS)
-    if (backend == QLatin1String("gdi"))
+    if (backend == u"gdi")
         return new QGdiWindowCapture;
 #if QT_CONFIG(cpp_winrt)
-    if (backend == QLatin1String("uwp"))
+    if (backend == u"uwp")
         return new QFFmpegWindowCaptureUwp;
 #endif
 #elif defined(Q_OS_MACOS)
-    if (backend == QLatin1String("cg"))
+    if (backend == u"cg")
         return new QCGWindowCapture;
 #endif
     return nullptr;
 }
 
 QFFmpegMediaIntegration::QFFmpegMediaIntegration()
+    : QPlatformMediaIntegration(QLatin1String("ffmpeg"))
 {
-    resolveSymbols();
-
     setupFFmpegLogger();
 
-#ifndef QT_NO_DEBUG
-    qDebug() << "Available HW decoding frameworks:";
-    for (auto type : QFFmpeg::HWAccel::decodingDeviceTypes())
-        qDebug() << "    " << av_hwdevice_get_type_name(type);
+    qCInfo(qLcFFmpeg) << "Using Qt multimedia with FFmpeg version" << av_version_info()
+                      << avutil_license();
 
-    qDebug() << "Available HW encoding frameworks:";
+    qCDebug(qLcFFmpeg) << "Available HW decoding frameworks:";
+    for (auto type : QFFmpeg::HWAccel::decodingDeviceTypes())
+        qCDebug(qLcFFmpeg) << "    " << av_hwdevice_get_type_name(type);
+
+    qCDebug(qLcFFmpeg) << "Available HW encoding frameworks:";
     for (auto type : QFFmpeg::HWAccel::encodingDeviceTypes())
-        qDebug() << "    " << av_hwdevice_get_type_name(type);
-#endif
+        qCDebug(qLcFFmpeg) << "    " << av_hwdevice_get_type_name(type);
 }
 
 QMaybe<QPlatformAudioDecoder *> QFFmpegMediaIntegration::createAudioDecoder(QAudioDecoder *decoder)
@@ -190,9 +181,11 @@ QMaybe<QPlatformAudioDecoder *> QFFmpegMediaIntegration::createAudioDecoder(QAud
     return new QFFmpegAudioDecoder(decoder);
 }
 
-QMaybe<QPlatformAudioResampler *> QFFmpegMediaIntegration::createAudioResampler(const QAudioFormat &inputFormat, const QAudioFormat &outputFormat)
+QMaybe<std::unique_ptr<QPlatformAudioResampler>>
+QFFmpegMediaIntegration::createAudioResampler(const QAudioFormat &inputFormat,
+                                              const QAudioFormat &outputFormat)
 {
-    return new QFFmpegResampler(inputFormat, outputFormat);
+    return { std::make_unique<QFFmpegResampler>(inputFormat, outputFormat) };
 }
 
 QMaybe<QPlatformMediaCaptureSession *> QFFmpegMediaIntegration::createCaptureSession()
@@ -223,7 +216,8 @@ QMaybe<QPlatformCamera *> QFFmpegMediaIntegration::createCamera(QCamera *camera)
 
 QPlatformSurfaceCapture *QFFmpegMediaIntegration::createScreenCapture(QScreenCapture *)
 {
-    static const QString screenCaptureBackend = qgetenv("QT_SCREEN_CAPTURE_BACKEND").toLower();
+    static const QString screenCaptureBackend =
+            QString::fromLocal8Bit(qgetenv("QT_SCREEN_CAPTURE_BACKEND")).toLower();
 
     if (!screenCaptureBackend.isEmpty()) {
         if (auto screenCapture = createScreenCaptureByBackend(screenCaptureBackend))
@@ -237,6 +231,11 @@ QPlatformSurfaceCapture *QFFmpegMediaIntegration::createScreenCapture(QScreenCap
         return new QX11SurfaceCapture(QPlatformSurfaceCapture::ScreenSource{});
 #endif
 
+#if QT_CONFIG(pipewire_screencapture)
+    if (QtPipeWire::QPipeWireCapture::isSupported())
+        return new QtPipeWire::QPipeWireCapture(QPlatformSurfaceCapture::ScreenSource{});
+#endif
+
 #if QT_CONFIG(eglfs)
     if (QEglfsScreenCapture::isSupported())
         return new QEglfsScreenCapture;
@@ -246,6 +245,8 @@ QPlatformSurfaceCapture *QFFmpegMediaIntegration::createScreenCapture(QScreenCap
     return new QFFmpegScreenCaptureDxgi;
 #elif defined(Q_OS_MACOS) // TODO: probably use it for iOS as well
     return new QAVFScreenCapture;
+#elif defined(Q_OS_ANDROID)
+    return new QAndroidScreenCapture;
 #else
     return new QGrabWindowSurfaceCapture(QPlatformSurfaceCapture::ScreenSource{});
 #endif
@@ -253,7 +254,8 @@ QPlatformSurfaceCapture *QFFmpegMediaIntegration::createScreenCapture(QScreenCap
 
 QPlatformSurfaceCapture *QFFmpegMediaIntegration::createWindowCapture(QWindowCapture *)
 {
-    static const QString windowCaptureBackend = qgetenv("QT_WINDOW_CAPTURE_BACKEND").toLower();
+    static const QString windowCaptureBackend =
+            QString::fromLocal8Bit(qgetenv("QT_WINDOW_CAPTURE_BACKEND")).toLower();
 
     if (!windowCaptureBackend.isEmpty()) {
         if (auto windowCapture = createWindowCaptureByBackend(windowCaptureBackend))
@@ -305,6 +307,12 @@ QMaybe<QPlatformAudioInput *> QFFmpegMediaIntegration::createAudioInput(QAudioIn
     return new QFFmpegAudioInput(input);
 }
 
+QVideoFrame QFFmpegMediaIntegration::convertVideoFrame(QVideoFrame &srcFrame,
+                                                       const QVideoFrameFormat &destFormat)
+{
+    return convertFrame(srcFrame, destFormat);
+}
+
 QPlatformMediaFormatInfo *QFFmpegMediaIntegration::createFormatInfo()
 {
     return new QFFmpegMediaFormatInfo;
@@ -317,7 +325,9 @@ QPlatformVideoDevices *QFFmpegMediaIntegration::createVideoDevices()
 #elif QT_CONFIG(linux_v4l)
     return new QV4L2CameraDevices(this);
 #elif defined Q_OS_DARWIN
-    return new QAVFVideoDevices(this);
+    return new QAVFVideoDevices(
+        this,
+        &QFFmpeg::isCVFormatSupported);
 #elif defined(Q_OS_WINDOWS)
     return new QWindowsVideoDevices(this);
 #else
@@ -356,13 +366,13 @@ Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void * /*reserved*/)
     if (av_jni_set_java_vm(vm, nullptr))
         return JNI_ERR;
 
-    if (!QAndroidCamera::registerNativeMethods())
+    if (!QAndroidCamera::registerNativeMethods()
+            ||!QAndroidScreenCapture::registerNativeMethods()) {
         return JNI_ERR;
+    }
 
     return JNI_VERSION_1_6;
 }
 #endif
 
 QT_END_NAMESPACE
-
-#include "qffmpegmediaintegration.moc"
